@@ -1,4 +1,3 @@
-import asyncio
 import inspect
 from multiprocessing import Process
 import os
@@ -13,6 +12,7 @@ currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentfram
 parentdir = os.path.dirname(currentdir)
 sys.path.insert(0, parentdir) 
 
+import hashlib
 import logging
 
 from abdi_config import AbdiConfig, LOGGER_NAME
@@ -38,7 +38,10 @@ class HolonicAgent(Agent, BrokerNotifier) :
         i = i or HolonicIntention()
         super().__init__(b, d, i)
         
-        self.agent_id = None
+        # self.agent_id = None
+        # self.short_id = None
+        self.agent_id = str(uuid.uuid4()).replace("-", "")
+        self.short_id = self.agent_id[:4] #hashlib.md5(self.agent_id.encode()).hexdigest()[:6]
         self.config = config if config else AbdiConfig(options={})
         self.head_agents = []
         self.body_agents = []
@@ -46,9 +49,10 @@ class HolonicAgent(Agent, BrokerNotifier) :
         
         self.name = f'<{self.__class__.__name__}>'
         self._agent_proc = None        
+        self._agent_thread = None        
         self._broker = None
         self._topic_handlers = {}
-        self._logistics = None
+        # self._logistics = None
 
 
     @final
@@ -65,14 +69,24 @@ class HolonicAgent(Agent, BrokerNotifier) :
             try:
                 self._agent_proc.join()
             except:
-                logger.warning(f"{self.name} terminated.")
+                logger.warning(f"{self.short_id}> {self.name} terminated.")
 
-    
-    # def unpack(self, payload):
-    #     if self._request_logistic.is_managed(payload):
-    #         return self._request_logistic.unpack(payload)
-    #     else:
-    #         return payload
+
+    @final
+    def start_thread(self, head=False):
+        self._agent_thread = threading.Thread(target=self._run, args=(self.config,))
+        self._agent_thread.start()
+        
+        for a in self.head_agents:
+            a.start()
+        for a in self.body_agents:
+            a.start()
+        
+        if head:
+            try:
+                self._agent_thread.join()
+            except:
+                logger.warning(f"{self.short_id}> {self.name} terminated.")
 
 
 # =====================
@@ -80,9 +94,9 @@ class HolonicAgent(Agent, BrokerNotifier) :
 # =====================
         
         
-    def append_logistic(self, logistic:BaseLogistic):
-        self._logistics.append(logistic)
-        return self._logistics
+    # def append_logistic(self, logistic:BaseLogistic):
+    #     self._logistics.append(logistic)
+    #     return self._logistics
 
 
     def is_running(self):
@@ -100,22 +114,25 @@ class HolonicAgent(Agent, BrokerNotifier) :
         self.on_begining()
 
         def signal_handler(signal, frame):
-            logger.warning(f"{self.name} Ctrl-C: {self.__class__.__name__}")
+            logger.warning(f"{self.short_id}> {self.name} Ctrl-C: {self.__class__.__name__}")
             self.terminate()
-        signal.signal(signal.SIGINT, signal_handler)
+        if self._agent_proc:
+            signal.signal(signal.SIGINT, signal_handler)
 
-        self.agent_id = str(uuid.uuid1()).replace("-", "")
+        # self.agent_id = str(uuid.uuid1()).replace("-", "")
+        # self.short_id = hashlib.md5(self.agent_id.encode()).hexdigest()[:6]
+        # logger.debug(f"agent_id: {self.agent_id}, short_id: {self.short_id}")
         self._terminate_lock = threading.Event()
-        self._logistics = []
+        # self._logistics = []
         
-        logger.debug(f"create broker")
+        logger.debug(f"{self.short_id}> Create broker")
         if broker_type := self.config.get_broker_type():
             self._broker = BrokerMaker().create_broker(
                 broker_type=broker_type, 
                 notifier=self)
             self._broker.start(options=self.config.options)
         
-        logger.debug(f"start interval_loop")
+        logger.debug(f"{self.short_id}> start interval_loop")
         def interval_loop():
             while not self._terminate_lock.is_set():
                 self.on_interval()
@@ -180,18 +197,18 @@ class HolonicAgent(Agent, BrokerNotifier) :
 
     @final
     def publish(self, topic, payload=None):
-        logistic_topic = topic
-        packed_payload = payload
+        # logistic_topic = topic
+        # packed_payload = payload
         
-        for logistic in self._logistics:
-            logistic_topic, packed_payload = logistic.pack(logistic_topic, packed_payload)
+        # for logistic in self._logistics:
+        #     logistic_topic, packed_payload = logistic.pack(logistic_topic, packed_payload)
             
-        logger.debug(f"logistic_topic: {logistic_topic}, packed_payload: {packed_payload}")
-        return self._broker.publish(logistic_topic, packed_payload)
+        # logger.debug(f"logistic_topic: {logistic_topic}, packed_payload: {packed_payload}")
+        # return self._broker.publish(logistic_topic, packed_payload)
         # logistic = self.get_logistic(topic)
         # packed = logistic.pack(payload)
         # return self._publish(topic, packed)
-        # return self._broker.publish(topic, payload)
+        return self._broker.publish(topic, payload)
 
 
     @final
@@ -199,16 +216,16 @@ class HolonicAgent(Agent, BrokerNotifier) :
         return self._broker.publish(topic, payload)
         
         
-    @final
-    def request(self, topic, payload):
-        # wrapped = self._request_logistic.pack(payload)
-        # self.publish(topic, wrapped)
-        self.publish(f"@request.{topic}", payload)
+    # @final
+    # def request(self, topic, payload):
+    #     # wrapped = self._request_logistic.pack(payload)
+    #     # self.publish(topic, wrapped)
+    #     self.publish(f"@request.{topic}", payload)
         
     
     def _on_response(self, topic, payload):
         managed_payload = self._request_logistic.unpack(payload)
-        logger.debug(f"receiver: {managed_payload['receiver']}, agent_id: {self.agent_id}")
+        logger.debug(f"{self.short_id}> receiver: {managed_payload['receiver']}, agent_id: {self.agent_id}")
         if managed_payload["receiver"] == self.agent_id:
             self._on_message(topic, managed_payload["content"])
 
@@ -218,14 +235,14 @@ class HolonicAgent(Agent, BrokerNotifier) :
         
     
     def _on_request(self, topic, payload):
-        logger.debug(f"topic: {topic}, payload: {payload}")
+        logger.debug(f"{self.short_id}> topic: {topic}, payload: {payload}")
         request_payload = self._request_logistic.unpack(payload)
 
         handler = self._topic_handlers[topic] if topic in self._topic_handlers else self.on_request
         response_topic, response_payload = handler(topic, request_payload["content"])
-        logger.debug(f"response_topic: {response_topic}")
+        logger.debug(f"{self.short_id}> response_topic: {response_topic}")
         if not response_payload:
-            logger.warning(f"response_payload is None or empty.")
+            logger.warning(f"{self.short_id}> response_payload is None or empty.")
         
         if response_topic:
             response_payload = self._request_logistic.pack_for_response(response_payload, request_payload)
@@ -236,18 +253,18 @@ class HolonicAgent(Agent, BrokerNotifier) :
     @final
     def subscribe(self, topic, data_type="str", topic_handler=None):
         if topic_handler:
-            self.register_topic_handler(topic, topic_handler)
+            self.set_topic_handler(topic, topic_handler)
         return self._broker.subscribe(topic, data_type)
     
     
-    def register_topic_handler(self, topic, handler):
-        logger.debug(f"Add topic handler: {topic}")
+    def set_topic_handler(self, topic, handler):
+        logger.debug(f"{self.short_id}> Set topic handler: {topic}")
         self._topic_handlers[topic] = handler
         
 
     @final
     def terminate(self):
-        logger.warn(f"{self.name}.")
+        logger.warn(f"{self.short_id}> {self.name}.")
 
         for a in self.head_agents:
             name = a.__class__.__name__
@@ -267,7 +284,7 @@ class HolonicAgent(Agent, BrokerNotifier) :
 
     
     def _on_connect(self):
-        logger.info(f"{self.name} Broker is connected.")
+        logger.info(f"{self.short_id}> {self.name} Broker is connected.")
         
         self.subscribe("echo")
         self.subscribe("terminate")
@@ -282,8 +299,10 @@ class HolonicAgent(Agent, BrokerNotifier) :
     def _on_message(self, topic:str, payload):
         # logger.debug(f"topic: {topic}, payload: {payload}")
         if topic in self._topic_handlers:
+            # logger.debug(f"topic in self._topic_handlers")
             self._topic_handlers[topic](topic, payload)
         else:
+            # logger.debug(f"on_message")
             self.on_message(topic, payload)
         
         
@@ -327,7 +346,7 @@ class HolonicAgent(Agent, BrokerNotifier) :
         try:
             data = payload.decode('utf-8', 'ignore')
         except Exception as ex:
-            logger.error(f"Type: {type(ex)}")
+            logger.error(f"{self.short_id}> Type: {type(ex)}")
             logger.exception(ex)
             data = ""
             
